@@ -1,5 +1,6 @@
 """This module contains a storage class and codec to support the N5 format.
 """
+
 import os
 import struct
 import sys
@@ -124,7 +125,11 @@ class N5Store(NestedDirectoryStore):
 
             for k in n5_keywords:
                 if k in zarr_attrs:
-                    warnings.warn(f"Attribute {k} is a reserved N5 keyword", UserWarning)
+                    warnings.warn(
+                        f"Attribute {k} is a reserved N5 keyword",
+                        UserWarning,
+                        stacklevel=2,
+                    )
 
             # remove previous user attributes
             for k in list(n5_attrs.keys()):
@@ -326,7 +331,10 @@ class N5FSStore(FSStore):
     def __init__(self, *args, **kwargs):
         if "dimension_separator" in kwargs:
             kwargs.pop("dimension_separator")
-            warnings.warn("Keyword argument `dimension_separator` will be ignored")
+            warnings.warn(
+                "Keyword argument `dimension_separator` will be ignored",
+                stacklevel=2,
+            )
         dimension_separator = "."
         super().__init__(*args, dimension_separator=dimension_separator, **kwargs)
 
@@ -410,7 +418,11 @@ class N5FSStore(FSStore):
 
             for k in n5_keywords:
                 if k in zarr_attrs.keys():
-                    warnings.warn(f"Attribute {k} is a reserved N5 keyword", UserWarning)
+                    warnings.warn(
+                        f"Attribute {k} is a reserved N5 keyword",
+                        UserWarning,
+                        stacklevel=2,
+                    )
 
             # replace previous user attributes
             for k in list(n5_attrs.keys()):
@@ -596,8 +608,8 @@ def array_metadata_to_n5(array_metadata: Dict[str, Any], top_level=False) -> Dic
         array_metadata["n5"] = N5_FORMAT
     try:
         dtype = np.dtype(array_metadata["dataType"])
-    except TypeError:
-        raise TypeError(f"Data type {array_metadata['dataType']} is not supported by N5")
+    except TypeError as e:
+        raise TypeError(f"Data type {array_metadata['dataType']} is not supported by N5") from e
 
     array_metadata["dataType"] = dtype.name
     array_metadata["dimensions"] = array_metadata["dimensions"][::-1]
@@ -710,6 +722,7 @@ def compressor_config_to_n5(compressor_config: Optional[Dict[str, Any]]) -> Dict
                 "Not all N5 implementations support lzma compression (yet). You "
                 "might not be able to open the dataset with another N5 library.",
                 RuntimeWarning,
+                stacklevel=2,
             )
             n5_config["format"] = _compressor_config["format"]
             n5_config["check"] = _compressor_config["check"]
@@ -825,9 +838,9 @@ class N5ChunkWrapper(Codec):
 
         if out is not None:
             # out should only be used if we read a complete chunk
-            assert chunk_shape == self.chunk_shape, "Expected chunk of shape {}, found {}".format(
-                self.chunk_shape, chunk_shape
-            )
+            assert (
+                chunk_shape == self.chunk_shape
+            ), f"Expected chunk of shape {self.chunk_shape}, found {chunk_shape}"
 
             if self._compressor:
                 self._compressor.decode(chunk, out)
