@@ -74,7 +74,6 @@ def _init_creation_kwargs(zarr_version, at_root=True):
 @pytest.mark.parametrize("zarr_version", _VERSIONS)
 @pytest.mark.parametrize("at_root", [False, True])
 def test_array(zarr_version, at_root):
-
     expected_zarr_version = DEFAULT_ZARR_VERSION if zarr_version is None else zarr_version
     kwargs = _init_creation_kwargs(zarr_version, at_root)
 
@@ -213,7 +212,6 @@ def test_full_additional_dtypes(zarr_version):
 @pytest.mark.parametrize("zarr_version", _VERSIONS)
 @pytest.mark.parametrize("at_root", [False, True])
 def test_open_array(zarr_version, at_root, dimension_separator):
-
     store = "data/array.zarr"
     kwargs = _init_creation_kwargs(zarr_version, at_root)
 
@@ -329,7 +327,6 @@ def test_open_array(zarr_version, at_root, dimension_separator):
 
 
 def test_open_array_none():
-
     # open with both store and zarr_version = None
     z = open_array(mode="w", shape=100, chunks=10)
     assert isinstance(z, Array)
@@ -339,7 +336,6 @@ def test_open_array_none():
 @pytest.mark.parametrize("dimension_separator", [".", "/", None])
 @pytest.mark.parametrize("zarr_version", _VERSIONS2)
 def test_open_array_infer_separator_from_store(zarr_version, dimension_separator):
-
     if zarr_version == 3:
         StoreClass = DirectoryStoreV3
         path = "data"
@@ -370,7 +366,6 @@ def test_open_array_infer_separator_from_store(zarr_version, dimension_separator
 # TODO: N5 support for v3
 @pytest.mark.parametrize("zarr_version", [None, 2])
 def test_open_array_n5(zarr_version):
-
     store = "data/array.zarr"
     kwargs = _init_creation_kwargs(zarr_version)
 
@@ -409,7 +404,6 @@ def test_open_array_n5(zarr_version):
 @pytest.mark.parametrize("zarr_version", _VERSIONS)
 @pytest.mark.parametrize("at_root", [False, True])
 def test_open_array_dict_store(zarr_version, at_root):
-
     # dict will become a KVStore
     store = dict()
     kwargs = _init_creation_kwargs(zarr_version, at_root)
@@ -503,7 +497,6 @@ def test_empty_like(zarr_version, at_root):
 @pytest.mark.parametrize("zarr_version", _VERSIONS)
 @pytest.mark.parametrize("at_root", [False, True])
 def test_zeros_like(zarr_version, at_root):
-
     kwargs = _init_creation_kwargs(zarr_version, at_root)
     expected_zarr_version = DEFAULT_ZARR_VERSION if zarr_version is None else zarr_version
 
@@ -529,7 +522,6 @@ def test_zeros_like(zarr_version, at_root):
 @pytest.mark.parametrize("zarr_version", _VERSIONS)
 @pytest.mark.parametrize("at_root", [False, True])
 def test_ones_like(zarr_version, at_root):
-
     kwargs = _init_creation_kwargs(zarr_version, at_root)
     expected_zarr_version = DEFAULT_ZARR_VERSION if zarr_version is None else zarr_version
 
@@ -556,7 +548,6 @@ def test_ones_like(zarr_version, at_root):
 @pytest.mark.parametrize("zarr_version", _VERSIONS)
 @pytest.mark.parametrize("at_root", [False, True])
 def test_full_like(zarr_version, at_root):
-
     kwargs = _init_creation_kwargs(zarr_version, at_root)
     expected_zarr_version = DEFAULT_ZARR_VERSION if zarr_version is None else zarr_version
 
@@ -757,3 +748,27 @@ def test_create_with_storage_transformers(at_root):
     z = create(1000000000, chunks=True, storage_transformers=[transformer], **kwargs)
     assert isinstance(z.chunk_store, DummyStorageTransfomer)
     assert z.chunk_store.test_value == DummyStorageTransfomer.TEST_CONSTANT
+
+
+@pytest.mark.parametrize(
+    ("init_shape", "init_chunks", "shape", "chunks"),
+    (
+        ((1,), (1,), (1,), (1,)),
+        ((1.0,), (1.0,), (1,), (1,)),
+        ((1.0,), False, (1,), (1,)),
+        ((1.0,), True, (1,), (1,)),
+        ((1.0,), None, (1,), (1,)),
+    ),
+)
+def test_shape_chunk_ints(init_shape, init_chunks, shape, chunks):
+    g = open_group()
+    array = g.create_dataset("ds", shape=init_shape, chunks=init_chunks, dtype=np.uint8)
+
+    assert all(
+        isinstance(s, int) for s in array.shape
+    ), f"Expected shape to be all ints but found {array.shape=}."
+    assert all(
+        isinstance(c, int) for c in array.chunks
+    ), f"Expected chunks to be all ints but found {array.chunks=}."
+    assert array.shape == shape, f"Expected {shape=} but found {array.shape=}."
+    assert array.chunks == chunks, f"Expected {chunks=} but found {array.chunks=}."
