@@ -5,7 +5,7 @@ import json
 import numpy as np
 import pytest
 
-from zarr.codecs import Blosc, Delta, Pickle, Zlib
+from zarr.codecs import Blosc, Delta, Pickle, Zlib, Zstd
 from zarr.errors import MetadataError
 from zarr.meta import (
     ZARR_FORMAT,
@@ -44,19 +44,16 @@ def test_encode_decode_array_1():
         order="C",
     )
 
-    meta_json = (
-        """{
+    meta_json = f"""{{
         "chunks": [10],
-        "compressor": {"id": "zlib", "level": 1},
+        "compressor": {{"id": "zlib", "level": 1}},
         "dtype": "<f8",
         "fill_value": null,
         "filters": null,
         "order": "C",
         "shape": [100],
-        "zarr_format": %s
-    }"""
-        % ZARR_FORMAT
-    )
+        "zarr_format": {ZARR_FORMAT}
+    }}"""
 
     # test encoding
     meta_enc = encode_array_metadata(meta)
@@ -90,27 +87,24 @@ def test_encode_decode_array_2():
         filters=[df.get_config()],
     )
 
-    meta_json = (
-        """{
+    meta_json = f"""{{
         "chunks": [10, 10],
-        "compressor": {
+        "compressor": {{
             "id": "blosc",
             "clevel": 3,
             "cname": "lz4",
             "shuffle": 2,
             "blocksize": 0
-        },
+        }},
         "dtype": [["a", "<i4"], ["b", "|S10"]],
         "fill_value": "AAAAAAAAAAAAAAAAAAA=",
         "filters": [
-            {"id": "delta", "astype": "<u2", "dtype": "|V14"}
+            {{"id": "delta", "astype": "<u2", "dtype": "|V14"}}
         ],
         "order": "F",
         "shape": [100, 100],
-        "zarr_format": %s
-    }"""
-        % ZARR_FORMAT
-    )
+        "zarr_format": {ZARR_FORMAT}
+    }}"""
 
     # test encoding
     meta_enc = encode_array_metadata(meta)
@@ -144,26 +138,22 @@ def test_encode_decode_array_complex():
             filters=[],
         )
 
-        meta_json = """{
+        meta_json = f"""{{
             "chunks": [10, 10],
-            "compressor": {
+            "compressor": {{
                 "id": "blosc",
                 "clevel": 3,
                 "cname": "lz4",
                 "shuffle": 2,
                 "blocksize": 0
-            },
-            "dtype": "%s",
+            }},
+            "dtype": "{dtype.str}",
             "fill_value": ["NaN", -1.0],
             "filters": [],
-            "order": "%s",
+            "order": "{dtype.char}",
             "shape": [100, 100],
-            "zarr_format": %s
-        }""" % (
-            dtype.str,
-            dtype.char,
-            ZARR_FORMAT,
-        )
+            "zarr_format": {ZARR_FORMAT}
+        }}"""
 
         # test encoding
         meta_enc = encode_array_metadata(meta)
@@ -201,26 +191,22 @@ def test_encode_decode_array_datetime_timedelta():
             filters=[],
         )
 
-        meta_json = """{
+        meta_json = f"""{{
             "chunks": [10, 10],
-            "compressor": {
+            "compressor": {{
                 "id": "blosc",
                 "clevel": 3,
                 "cname": "lz4",
                 "shuffle": 2,
                 "blocksize": 0
-            },
-            "dtype": "%s",
+            }},
+            "dtype": "{dtype.str}",
             "fill_value": -9223372036854775808,
             "filters": [],
-            "order": "%s",
+            "order": "{dtype.char}",
             "shape": [100, 100],
-            "zarr_format": %s
-        }""" % (
-            dtype.str,
-            dtype.char,
-            ZARR_FORMAT,
-        )
+            "zarr_format": {ZARR_FORMAT}
+        }}"""
 
         # test encoding
         meta_enc = encode_array_metadata(meta)
@@ -253,19 +239,16 @@ def test_encode_decode_array_dtype_shape():
         order="C",
     )
 
-    meta_json = (
-        """{
+    meta_json = f"""{{
         "chunks": [10],
-        "compressor": {"id": "zlib", "level": 1},
+        "compressor": {{"id": "zlib", "level": 1}},
         "dtype": "<f8",
         "fill_value": null,
         "filters": null,
         "order": "C",
         "shape": [100, 10, 10],
-        "zarr_format": %s
-    }"""
-        % ZARR_FORMAT
-    )
+        "zarr_format": {ZARR_FORMAT}
+    }}"""
 
     # test encoding
     meta_enc = encode_array_metadata(meta)
@@ -285,17 +268,23 @@ def test_encode_decode_array_dtype_shape():
     assert meta_dec["filters"] is None
 
 
-def test_encode_decode_array_dtype_shape_v3():
+@pytest.mark.parametrize("cname", ["zlib", "zstd"])
+def test_encode_decode_array_dtype_shape_v3(cname):
+    if cname == "zlib":
+        compressor = Zlib(1)
+    elif cname == "zstd":
+        compressor = Zstd(1)
     meta = dict(
         shape=(100,),
         chunk_grid=dict(type="regular", chunk_shape=(10,), separator=("/")),
         data_type=np.dtype("(10, 10)<f8"),
-        compressor=Zlib(1),
+        compressor=compressor,
         fill_value=None,
         chunk_memory_layout="C",
     )
 
-    meta_json = """{
+    meta_json = (
+        """{
         "attributes": {},
         "chunk_grid": {
             "chunk_shape": [10],
@@ -304,7 +293,11 @@ def test_encode_decode_array_dtype_shape_v3():
         },
         "chunk_memory_layout": "C",
         "compressor": {
-            "codec": "https://purl.org/zarr/spec/codec/zlib/1.0",
+        """
+        + f"""
+        "codec": "https://purl.org/zarr/spec/codec/{cname}/1.0",
+        """
+        + """
             "configuration": {
                 "level": 1
             }
@@ -314,6 +307,7 @@ def test_encode_decode_array_dtype_shape_v3():
         "fill_value": null,
         "shape": [100, 10, 10 ]
     }"""
+    )
 
     # test encoding
     meta_enc = Metadata3.encode_array_metadata(meta)
@@ -332,7 +326,7 @@ def test_encode_decode_array_dtype_shape_v3():
     assert "filters" not in meta_dec
 
 
-@pytest.mark.parametrize("comp_id", ["gzip", "zlib", "blosc", "bz2", "lz4", "lzma"])
+@pytest.mark.parametrize("comp_id", ["gzip", "zlib", "blosc", "bz2", "lz4", "lzma", "zstd"])
 def test_decode_metadata_implicit_compressor_config_v3(comp_id):
     meta = {
         "attributes": {},
@@ -367,19 +361,16 @@ def test_encode_decode_array_structured():
         order="C",
     )
 
-    meta_json = (
-        """{
+    meta_json = f"""{{
         "chunks": [10],
-        "compressor": {"id": "zlib", "level": 1},
+        "compressor": {{"id": "zlib", "level": 1}},
         "dtype": [["f0", "<i8"], ["f1", "<f8", [10, 10]], ["f2", "|u1", [5, 10, 15]]],
         "fill_value": null,
         "filters": null,
         "order": "C",
         "shape": [100],
-        "zarr_format": %s
-    }"""
-        % ZARR_FORMAT
-    )
+        "zarr_format": {ZARR_FORMAT}
+    }}"""
 
     # test encoding
     meta_enc = encode_array_metadata(meta)
@@ -402,8 +393,8 @@ def test_encode_decode_array_structured():
 def test_encode_decode_fill_values_nan():
     fills = (
         (np.nan, "NaN", np.isnan),
-        (np.NINF, "-Infinity", np.isneginf),
-        (np.PINF, "Infinity", np.isposinf),
+        (-np.inf, "-Infinity", np.isneginf),
+        (np.inf, "Infinity", np.isposinf),
     )
 
     for v, s, f in fills:
@@ -417,19 +408,16 @@ def test_encode_decode_fill_values_nan():
             order="C",
         )
 
-        meta_json = """{
+        meta_json = f"""{{
             "chunks": [10],
-            "compressor": {"id": "zlib", "level": 1},
+            "compressor": {{"id": "zlib", "level": 1}},
             "dtype": "<f8",
-            "fill_value": "%s",
+            "fill_value": "{s}",
             "filters": null,
             "order": "C",
             "shape": [100],
-            "zarr_format": %s
-        }""" % (
-            s,
-            ZARR_FORMAT,
-        )
+            "zarr_format": {ZARR_FORMAT}
+        }}"""
 
         # test encoding
         meta_enc = encode_array_metadata(meta)
@@ -461,19 +449,16 @@ def test_encode_decode_fill_values_bytes():
         # define expected metadata encoded as JSON
         s = base64.standard_b64encode(v)
         s = s.decode()
-        meta_json = """{
+        meta_json = f"""{{
             "chunks": [10],
-            "compressor": {"id": "zlib", "level": 1},
+            "compressor": {{"id": "zlib", "level": 1}},
             "dtype": "|S10",
-            "fill_value": "%s",
+            "fill_value": "{s}",
             "filters": null,
             "order": "C",
             "shape": [100],
-            "zarr_format": %s
-        }""" % (
-            s,
-            ZARR_FORMAT,
-        )
+            "zarr_format": {ZARR_FORMAT}
+        }}"""
 
         # test encoding
         assert_json_equal(meta_json, meta_enc)
@@ -487,29 +472,24 @@ def test_encode_decode_fill_values_bytes():
 
 def test_decode_array_unsupported_format():
     # unsupported format
-    meta_json = """{
-        "zarr_format": %s,
+    meta_json = f"""{{
+        "zarr_format": {ZARR_FORMAT - 1},
         "shape": [100],
         "chunks": [10],
         "dtype": "<f8",
-        "compressor": {"id": "zlib", "level": 1},
+        "compressor": {{"id": "zlib", "level": 1}},
         "fill_value": null,
         "order": "C"
-    }""" % (
-        ZARR_FORMAT - 1
-    )
+    }}"""
     with pytest.raises(MetadataError):
         decode_array_metadata(meta_json)
 
 
 def test_decode_array_missing_fields():
     # missing fields
-    meta_json = (
-        """{
-        "zarr_format": %s
-    }"""
-        % ZARR_FORMAT
-    )
+    meta_json = f"""{{
+        "zarr_format": {ZARR_FORMAT}
+    }}"""
     with pytest.raises(MetadataError):
         decode_array_metadata(meta_json)
 
@@ -525,21 +505,16 @@ def test_encode_decode_dtype():
 
 def test_decode_group():
     # typical
-    b = (
-        """{
-        "zarr_format": %s
-    }"""
-        % ZARR_FORMAT
-    )
+    b = f"""{{
+        "zarr_format": {ZARR_FORMAT}
+    }}"""
     meta = decode_group_metadata(b)
     assert ZARR_FORMAT == meta["zarr_format"]
 
     # unsupported format
-    b = """{
-        "zarr_format": %s
-    }""" % (
-        ZARR_FORMAT - 1
-    )
+    b = f"""{{
+        "zarr_format": {ZARR_FORMAT - 1}
+    }}"""
     with pytest.raises(MetadataError):
         decode_group_metadata(b)
 
@@ -637,7 +612,7 @@ def test_metadata3_exceptions():
     required = ["zarr_format", "metadata_encoding", "metadata_key_suffix", "extensions"]
     for key in required:
         meta = copy.copy(_default_entry_point_metadata_v3)
-        meta.pop("zarr_format")
+        meta.pop(key)
         with pytest.raises(ValueError):
             # cannot encode metadata that is missing a required key
             Metadata3.encode_hierarchy_metadata(meta)
